"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildTree = exports.reactTreeToFlexTree = void 0;
const TestRenderer = __importStar(require("react-test-renderer"));
const yoga_layout_prebuilt_1 = __importDefault(require("yoga-layout-prebuilt"));
const Context_1 = require("./utils/Context");
const hasAnyDefined_1 = require("./utils/hasAnyDefined");
const pick_1 = require("./utils/pick");
const computeYogaTree_1 = require("./jsonUtils/computeYogaTree");
const computeTextTree_1 = require("./jsonUtils/computeTextTree");
const constants_1 = require("./utils/constants");
const zIndex_1 = require("./utils/zIndex");
exports.reactTreeToFlexTree = (node, yogaNode, context) => {
    let textNodes = [];
    let textStyle = context.getInheritedStyles();
    let newChildren = [];
    let style;
    let type;
    if (typeof node === 'string') {
        textNodes = computeTextTree_1.computeTextTree(node, context);
        type = 'sketch_text';
    }
    else {
        style = node.props && node.props.style ? node.props.style : {};
        type = node.type || 'sketch_text';
        if (type === 'sketch_svg' && node.children) {
            // @ts-ignore
            newChildren = node.children;
        }
        else if (type === 'sketch_text') {
            // If current node is a Text node, add text styles to Context to pass down to
            // child nodes.
            if (node.props && node.props.style && hasAnyDefined_1.hasAnyDefined(style, constants_1.INHERITABLE_FONT_STYLES)) {
                const inheritableStyles = pick_1.pick(style, constants_1.INHERITABLE_FONT_STYLES);
                inheritableStyles.flexDirection = 'row';
                context.addInheritableStyles(inheritableStyles);
                textStyle = Object.assign(Object.assign({}, context.getInheritedStyles()), inheritableStyles);
            }
            // Compute Text Children
            textNodes = computeTextTree_1.computeTextTree(node, context);
        }
        else if (node.children && node.children.length > 0) {
            // Recursion reverses the render stacking order
            // but that's actually fine because Sketch renders the first on top
            // Calculates zIndex order to match yoga
            const children = zIndex_1.zIndex(node.children);
            for (let index = 0; index < children.length; index += 1) {
                const childComponent = children[index];
                const childNode = yogaNode.getChild(index);
                const renderedChildComponent = exports.reactTreeToFlexTree(childComponent, childNode, context.forChildren());
                newChildren.push(renderedChildComponent);
            }
        }
    }
    return {
        type,
        style,
        textStyle,
        layout: {
            left: yogaNode.getComputedLeft(),
            right: yogaNode.getComputedRight(),
            top: yogaNode.getComputedTop(),
            bottom: yogaNode.getComputedBottom(),
            width: yogaNode.getComputedWidth(),
            height: yogaNode.getComputedHeight(),
        },
        props: Object.assign(Object.assign({}, (typeof node !== 'string' ? node.props : {})), { textNodes }),
        children: newChildren,
    };
};
exports.buildTree = (bridge) => (element) => {
    let renderer;
    if (typeof TestRenderer.act !== 'undefined') {
        TestRenderer.act(() => {
            // synchronous callback
            renderer = TestRenderer.create(element);
        });
    }
    else {
        renderer = TestRenderer.create(element);
    }
    if (!renderer) {
        throw new Error('Cannot access react renderer');
    }
    const json = renderer.toJSON();
    if (!json) {
        throw new Error('Cannot render react element');
    }
    const yogaNode = computeYogaTree_1.computeYogaTree(bridge)(json, new Context_1.Context());
    yogaNode.calculateLayout(undefined, undefined, yoga_layout_prebuilt_1.default.DIRECTION_LTR);
    const tree = exports.reactTreeToFlexTree(json, yogaNode, new Context_1.Context());
    return tree;
};
