"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.flexToSketchJSON = void 0;
const renderers = __importStar(require("./renderers"));
function missingRendererError(type, annotations) {
    return new Error(`Could not find renderer for type '${type}'.${annotations ? `\n${annotations}` : ''}`);
}
exports.flexToSketchJSON = (bridge) => (node) => {
    if (typeof node === 'string') {
        throw missingRendererError('string');
    }
    const { type, children } = node;
    // Give some insight as to why there might be issues
    // specific to Page and Document components or SVG components
    if (type === 'sketch_document') {
        throw missingRendererError(type, 'Be sure to only have <Page> components as children of <Document>.');
    }
    // @ts-ignore
    const Renderer = renderers[type];
    if (Renderer == null) {
        if (type.indexOf('svg') === 0) {
            // the svg renderer should stop the walk down the tree so it shouldn't happen
            throw missingRendererError(type, 'Be sure to always have <Svg.*> components as children of <Svg>.');
        }
        throw missingRendererError(type);
    }
    const renderer = new Renderer(bridge);
    const groupLayer = renderer.renderGroupLayer(node);
    if (groupLayer._class === 'symbolInstance') {
        return groupLayer;
    }
    const backingLayers = renderer.renderBackingLayers(node);
    // stopping the walk down the tree if we have an svg
    const curriedFlexToSketchJSON = exports.flexToSketchJSON(bridge);
    const sublayers = children && type !== 'sketch_svg'
        ? children.map((child) => curriedFlexToSketchJSON(child))
        : [];
    // Filter out anything null, undefined
    const layers = [...backingLayers, ...sublayers].filter((l) => l);
    return Object.assign(Object.assign({}, groupLayer), { layers });
};
