"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createBorders = exports.createUniformBorder = exports.DEFAULT_BORDER_STYLE = void 0;
const sketch_file_format_ts_1 = require("@sketch-hq/sketch-file-format-ts");
const models_1 = require("./models");
const same_1 = require("../utils/same");
const shapeLayers_1 = require("./shapeLayers");
const style_1 = require("./style");
const DEFAULT_BORDER_COLOR = 'transparent';
exports.DEFAULT_BORDER_STYLE = 'solid';
exports.createUniformBorder = (width, color, style = exports.DEFAULT_BORDER_STYLE, position = sketch_file_format_ts_1.FileFormat1.BorderPosition.Center, lineCapStyle = sketch_file_format_ts_1.FileFormat1.LineCapStyle.Butt, lineJoinStyle = sketch_file_format_ts_1.FileFormat1.LineJoinStyle.Miter) => {
    const borderOptions = style_1.makeBorderOptions(style, width, lineCapStyle, lineJoinStyle);
    const borders = [
        {
            _class: 'border',
            isEnabled: true,
            color: models_1.makeColorFromCSS(color),
            fillType: sketch_file_format_ts_1.FileFormat1.FillType.Color,
            position,
            thickness: width,
            contextSettings: {
                _class: 'graphicsContextSettings',
                blendMode: sketch_file_format_ts_1.FileFormat1.BlendMode.Normal,
                opacity: 1,
            },
            gradient: models_1.emptyGradient,
        },
    ];
    return { borderOptions, borders };
};
exports.createBorders = (content, layout, style) => {
    if (!style) {
        return [content];
    }
    const { borderTopWidth = 0, borderRightWidth = 0, borderBottomWidth = 0, borderLeftWidth = 0, borderTopColor = DEFAULT_BORDER_COLOR, borderRightColor = DEFAULT_BORDER_COLOR, borderBottomColor = DEFAULT_BORDER_COLOR, borderLeftColor = DEFAULT_BORDER_COLOR, borderTopStyle = exports.DEFAULT_BORDER_STYLE, borderRightStyle = exports.DEFAULT_BORDER_STYLE, borderBottomStyle = exports.DEFAULT_BORDER_STYLE, borderLeftStyle = exports.DEFAULT_BORDER_STYLE, } = style;
    if (same_1.same(borderTopWidth, borderRightWidth, borderBottomWidth, borderLeftWidth) &&
        same_1.same(borderTopColor, borderRightColor, borderBottomColor, borderLeftColor) &&
        same_1.same(borderTopStyle, borderRightStyle, borderBottomStyle, borderLeftStyle)) {
        // all sides have same border width
        // in this case, we can do everything with just a single shape.
        if (borderTopStyle !== undefined && borderTopWidth !== null) {
            const borderOptions = style_1.makeBorderOptions(borderTopStyle, borderTopWidth);
            if (borderOptions && content.style) {
                content.style.borderOptions = borderOptions;
            }
        }
        if (borderTopWidth && borderTopWidth > 0 && content.style) {
            content.style.borders = exports.createUniformBorder(borderTopWidth, borderTopColor, 'solid', sketch_file_format_ts_1.FileFormat1.BorderPosition.Outside).borders;
            const backingLayer = content.layers ? content.layers[0] : undefined;
            if (backingLayer) {
                backingLayer.frame.x += borderTopWidth;
                backingLayer.frame.y += borderTopWidth;
                backingLayer.frame.width -= borderTopWidth * 2;
                backingLayer.frame.height -= borderTopWidth * 2;
            }
        }
        return [content];
    }
    content.hasClippingMask = true;
    const layers = [content];
    if (borderTopWidth && borderTopWidth > 0) {
        const topBorder = shapeLayers_1.makeHorizontalBorder(0, 0, layout.width, borderTopWidth, borderTopColor);
        topBorder.name = 'Border (top)';
        const borderOptions = style_1.makeBorderOptions(borderTopStyle, borderTopWidth);
        if (borderOptions && topBorder.style) {
            topBorder.style.borderOptions = borderOptions;
        }
        layers.push(topBorder);
    }
    if (borderRightWidth && borderRightWidth > 0) {
        const rightBorder = shapeLayers_1.makeVerticalBorder(layout.width - borderRightWidth, 0, layout.height, borderRightWidth, borderRightColor);
        rightBorder.name = 'Border (right)';
        const borderOptions = style_1.makeBorderOptions(borderRightStyle, borderRightWidth);
        if (borderOptions && rightBorder.style) {
            rightBorder.style.borderOptions = borderOptions;
        }
        layers.push(rightBorder);
    }
    if (borderBottomWidth && borderBottomWidth > 0) {
        const bottomBorder = shapeLayers_1.makeHorizontalBorder(0, layout.height - borderBottomWidth, layout.width, borderBottomWidth, borderBottomColor);
        bottomBorder.name = 'Border (bottom)';
        const borderOptions = style_1.makeBorderOptions(borderBottomStyle, borderBottomWidth);
        if (borderOptions && bottomBorder.style) {
            bottomBorder.style.borderOptions = borderOptions;
        }
        layers.push(bottomBorder);
    }
    if (borderLeftWidth && borderLeftWidth > 0) {
        const leftBorder = shapeLayers_1.makeVerticalBorder(0, 0, layout.height, borderLeftWidth, borderLeftColor);
        leftBorder.name = 'Border (left)';
        const borderOptions = style_1.makeBorderOptions(borderLeftStyle, borderLeftWidth);
        if (borderOptions && leftBorder.style) {
            leftBorder.style.borderOptions = borderOptions;
        }
        layers.push(leftBorder);
    }
    return layers;
};
