"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.computeYogaNode = exports.getStyles = void 0;
const yoga_layout_prebuilt_1 = __importDefault(require("yoga-layout-prebuilt"));
const createStringMeasurer_1 = require("../utils/createStringMeasurer");
const hasAnyDefined_1 = require("../utils/hasAnyDefined");
const pick_1 = require("../utils/pick");
const computeTextTree_1 = require("./computeTextTree");
const constants_1 = require("../utils/constants");
const isDefined_1 = require("../utils/isDefined");
const symbol_1 = require("../symbol");
// flatten all styles (including nested) into one object
exports.getStyles = (node) => {
    if (typeof node === 'string') {
        return {};
    }
    let { style } = node.props;
    if (Array.isArray(style)) {
        const flattened = Array.prototype.concat.apply([], style);
        const themeFlattened = Array.prototype.concat.apply([], flattened);
        const objectsOnly = themeFlattened.filter((f) => f);
        style = Object.assign({}, ...objectsOnly);
    }
    return style;
};
exports.computeYogaNode = (bridge) => (node, context) => {
    const yogaNode = yoga_layout_prebuilt_1.default.Node.create();
    const hasStyle = typeof node !== 'string' && node.props && node.props.style;
    const style = hasStyle ? exports.getStyles(node) : {};
    // Setup default symbol instance dimensions
    if (typeof node !== 'string' && node.type === 'sketch_symbolinstance') {
        const symbolProps = node.props;
        const symbolMaster = symbol_1.getSymbolMasterById(symbolProps.symbolID);
        if (!symbolMaster) {
            throw new Error('Cannot find Symbol Master with id ' + symbolProps.symbolID);
        }
        const { frame } = symbolMaster;
        yogaNode.setWidth(frame.width);
        yogaNode.setHeight(frame.height);
    }
    if (typeof node !== 'string' && node.type === 'sketch_svg') {
        const svgProps = node.props;
        // Width
        if (isDefined_1.isDefined(svgProps.width)) {
            yogaNode.setWidth(svgProps.width);
        }
        // Height
        if (isDefined_1.isDefined(svgProps.height)) {
            yogaNode.setHeight(svgProps.height);
        }
    }
    if (hasStyle) {
        // http://facebook.github.io/react-native/releases/0.48/docs/layout-props.html
        // Width
        if (isDefined_1.isDefined(style.width)) {
            yogaNode.setWidth(style.width);
        }
        // Height
        if (isDefined_1.isDefined(style.height)) {
            yogaNode.setHeight(style.height);
        }
        // Min-Height
        if (isDefined_1.isDefined(style.minHeight)) {
            yogaNode.setMinHeight(style.minHeight);
        }
        // Min-Width
        if (isDefined_1.isDefined(style.minWidth)) {
            yogaNode.setMinWidth(style.minWidth);
        }
        // Max-Height
        if (isDefined_1.isDefined(style.maxHeight)) {
            yogaNode.setMaxHeight(style.maxHeight);
        }
        // Min-Width
        if (isDefined_1.isDefined(style.maxWidth)) {
            yogaNode.setMaxWidth(style.maxWidth);
        }
        // Margin
        if (isDefined_1.isDefined(style.marginTop)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_TOP, style.marginTop);
        }
        if (isDefined_1.isDefined(style.marginBottom)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_BOTTOM, style.marginBottom);
        }
        if (isDefined_1.isDefined(style.marginLeft)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_LEFT, style.marginLeft);
        }
        if (isDefined_1.isDefined(style.marginRight)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_RIGHT, style.marginRight);
        }
        if (isDefined_1.isDefined(style.marginVertical)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_VERTICAL, style.marginVertical);
        }
        if (isDefined_1.isDefined(style.marginHorizontal)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_HORIZONTAL, style.marginHorizontal);
        }
        if (isDefined_1.isDefined(style.margin)) {
            yogaNode.setMargin(yoga_layout_prebuilt_1.default.EDGE_ALL, style.margin);
        }
        // Padding
        if (isDefined_1.isDefined(style.paddingTop)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_TOP, style.paddingTop);
        }
        if (isDefined_1.isDefined(style.paddingBottom)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_BOTTOM, style.paddingBottom);
        }
        if (isDefined_1.isDefined(style.paddingLeft)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_LEFT, style.paddingLeft);
        }
        if (isDefined_1.isDefined(style.paddingRight)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_RIGHT, style.paddingRight);
        }
        if (isDefined_1.isDefined(style.paddingVertical)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_VERTICAL, style.paddingVertical);
        }
        if (isDefined_1.isDefined(style.paddingHorizontal)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_HORIZONTAL, style.paddingHorizontal);
        }
        if (isDefined_1.isDefined(style.padding)) {
            yogaNode.setPadding(yoga_layout_prebuilt_1.default.EDGE_ALL, style.padding);
        }
        // Border
        if (isDefined_1.isDefined(style.borderTopWidth)) {
            yogaNode.setBorder(yoga_layout_prebuilt_1.default.EDGE_TOP, style.borderTopWidth);
        }
        if (isDefined_1.isDefined(style.borderBottomWidth)) {
            yogaNode.setBorder(yoga_layout_prebuilt_1.default.EDGE_BOTTOM, style.borderBottomWidth);
        }
        if (isDefined_1.isDefined(style.borderLeftWidth)) {
            yogaNode.setBorder(yoga_layout_prebuilt_1.default.EDGE_LEFT, style.borderLeftWidth);
        }
        if (isDefined_1.isDefined(style.borderRightWidth)) {
            yogaNode.setBorder(yoga_layout_prebuilt_1.default.EDGE_RIGHT, style.borderRightWidth);
        }
        if (isDefined_1.isDefined(style.borderWidth)) {
            yogaNode.setBorder(yoga_layout_prebuilt_1.default.EDGE_ALL, style.borderWidth);
        }
        // Flex
        if (isDefined_1.isDefined(style.flex)) {
            yogaNode.setFlex(style.flex);
        }
        if (isDefined_1.isDefined(style.flexGrow)) {
            yogaNode.setFlexGrow(style.flexGrow);
        }
        if (isDefined_1.isDefined(style.flexShrink)) {
            yogaNode.setFlexShrink(style.flexShrink);
        }
        if (isDefined_1.isDefined(style.flexBasis)) {
            yogaNode.setFlexBasis(style.flexBasis);
        }
        // Position
        if (style.position === 'absolute') {
            yogaNode.setPositionType(yoga_layout_prebuilt_1.default.POSITION_TYPE_ABSOLUTE);
        }
        if (style.position === 'relative') {
            yogaNode.setPositionType(yoga_layout_prebuilt_1.default.POSITION_TYPE_RELATIVE);
        }
        if (isDefined_1.isDefined(style.top)) {
            yogaNode.setPosition(yoga_layout_prebuilt_1.default.EDGE_TOP, style.top);
        }
        if (isDefined_1.isDefined(style.left)) {
            yogaNode.setPosition(yoga_layout_prebuilt_1.default.EDGE_LEFT, style.left);
        }
        if (isDefined_1.isDefined(style.right)) {
            yogaNode.setPosition(yoga_layout_prebuilt_1.default.EDGE_RIGHT, style.right);
        }
        if (isDefined_1.isDefined(style.bottom)) {
            yogaNode.setPosition(yoga_layout_prebuilt_1.default.EDGE_BOTTOM, style.bottom);
        }
        // Display
        if (style.display) {
            if (style.display === 'flex') {
                yogaNode.setDisplay(yoga_layout_prebuilt_1.default.DISPLAY_FLEX);
            }
            if (style.display === 'none') {
                yogaNode.setDisplay(yoga_layout_prebuilt_1.default.DISPLAY_NONE);
            }
        }
        // Overflow
        if (style.overflow) {
            if (style.overflow === 'visible') {
                yogaNode.setOverflow(yoga_layout_prebuilt_1.default.OVERFLOW_VISIBLE);
            }
            if (style.overflow === 'scroll') {
                yogaNode.setOverflow(yoga_layout_prebuilt_1.default.OVERFLOW_SCROLL);
            }
            if (style.overflow === 'hidden') {
                yogaNode.setOverflow(yoga_layout_prebuilt_1.default.OVERFLOW_HIDDEN);
            }
        }
        // Flex direction
        if (style.flexDirection) {
            if (style.flexDirection === 'row') {
                yogaNode.setFlexDirection(yoga_layout_prebuilt_1.default.FLEX_DIRECTION_ROW);
            }
            if (style.flexDirection === 'column') {
                yogaNode.setFlexDirection(yoga_layout_prebuilt_1.default.FLEX_DIRECTION_COLUMN);
            }
            if (style.flexDirection === 'row-reverse') {
                yogaNode.setFlexDirection(yoga_layout_prebuilt_1.default.FLEX_DIRECTION_ROW_REVERSE);
            }
            if (style.flexDirection === 'column-reverse') {
                yogaNode.setFlexDirection(yoga_layout_prebuilt_1.default.FLEX_DIRECTION_COLUMN_REVERSE);
            }
        }
        // Justify Content
        if (style.justifyContent) {
            if (style.justifyContent === 'flex-start') {
                yogaNode.setJustifyContent(yoga_layout_prebuilt_1.default.JUSTIFY_FLEX_START);
            }
            if (style.justifyContent === 'flex-end') {
                yogaNode.setJustifyContent(yoga_layout_prebuilt_1.default.JUSTIFY_FLEX_END);
            }
            if (style.justifyContent === 'center') {
                yogaNode.setJustifyContent(yoga_layout_prebuilt_1.default.JUSTIFY_CENTER);
            }
            if (style.justifyContent === 'space-between') {
                yogaNode.setJustifyContent(yoga_layout_prebuilt_1.default.JUSTIFY_SPACE_BETWEEN);
            }
            if (style.justifyContent === 'space-around') {
                yogaNode.setJustifyContent(yoga_layout_prebuilt_1.default.JUSTIFY_SPACE_AROUND);
            }
        }
        // Align Content
        if (style.alignContent) {
            if (style.alignContent === 'flex-start') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_FLEX_START);
            }
            if (style.alignContent === 'flex-end') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_FLEX_END);
            }
            if (style.alignContent === 'center') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_CENTER);
            }
            if (style.alignContent === 'stretch') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_STRETCH);
            }
            if (style.alignContent === 'baseline') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_BASELINE);
            }
            if (style.alignContent === 'space-between') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_SPACE_BETWEEN);
            }
            if (style.alignContent === 'space-around') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_SPACE_AROUND);
            }
            if (style.alignContent === 'auto') {
                yogaNode.setAlignContent(yoga_layout_prebuilt_1.default.ALIGN_AUTO);
            }
        }
        // Align Items
        if (style.alignItems) {
            if (style.alignItems === 'flex-start') {
                yogaNode.setAlignItems(yoga_layout_prebuilt_1.default.ALIGN_FLEX_START);
            }
            if (style.alignItems === 'flex-end') {
                yogaNode.setAlignItems(yoga_layout_prebuilt_1.default.ALIGN_FLEX_END);
            }
            if (style.alignItems === 'center') {
                yogaNode.setAlignItems(yoga_layout_prebuilt_1.default.ALIGN_CENTER);
            }
            if (style.alignItems === 'stretch') {
                yogaNode.setAlignItems(yoga_layout_prebuilt_1.default.ALIGN_STRETCH);
            }
            if (style.alignItems === 'baseline') {
                yogaNode.setAlignItems(yoga_layout_prebuilt_1.default.ALIGN_BASELINE);
            }
        }
        // Align Self
        if (style.alignSelf) {
            if (style.alignSelf === 'flex-start') {
                yogaNode.setAlignSelf(yoga_layout_prebuilt_1.default.ALIGN_FLEX_START);
            }
            if (style.alignSelf === 'flex-end') {
                yogaNode.setAlignSelf(yoga_layout_prebuilt_1.default.ALIGN_FLEX_END);
            }
            if (style.alignSelf === 'center') {
                yogaNode.setAlignSelf(yoga_layout_prebuilt_1.default.ALIGN_CENTER);
            }
            if (style.alignSelf === 'stretch') {
                yogaNode.setAlignSelf(yoga_layout_prebuilt_1.default.ALIGN_STRETCH);
            }
            if (style.alignSelf === 'baseline') {
                yogaNode.setAlignSelf(yoga_layout_prebuilt_1.default.ALIGN_BASELINE);
            }
        }
        // Flex Wrap
        if (style.flexWrap) {
            if (style.flexWrap === 'nowrap') {
                yogaNode.setFlexWrap(yoga_layout_prebuilt_1.default.WRAP_NO_WRAP);
            }
            if (style.flexWrap === 'wrap') {
                yogaNode.setFlexWrap(yoga_layout_prebuilt_1.default.WRAP_WRAP);
            }
            if (style.flexWrap === 'wrap-reverse') {
                yogaNode.setFlexWrap(yoga_layout_prebuilt_1.default.WRAP_WRAP_REVERSE);
            }
        }
    }
    if (typeof node === 'string' || node.type === 'sketch_text') {
        // If current node is a Text node, add text styles to Context to pass down to
        // child nodes.
        if (typeof node !== 'string' &&
            node.props &&
            node.props.style &&
            hasAnyDefined_1.hasAnyDefined(style, constants_1.INHERITABLE_FONT_STYLES)) {
            // @ts-ignore
            const inheritableStyles = pick_1.pick(style, constants_1.INHERITABLE_FONT_STYLES);
            context.addInheritableStyles(inheritableStyles);
        }
        // Handle Text Children
        const textNodes = computeTextTree_1.computeTextTree(node, context);
        yogaNode.setMeasureFunc(createStringMeasurer_1.createStringMeasurer(bridge)(textNodes));
        return { node: yogaNode, stop: true };
    }
    return { node: yogaNode };
};
