"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeHorizontalBorder = exports.makeVerticalBorder = exports.makeShapeGroup = exports.makeRectShapeLayer = exports.makeShapePath = exports.makeRectPath = exports.makeVerticalPath = exports.makeHorizontalPath = void 0;
const sketch_file_format_ts_1 = require("@sketch-hq/sketch-file-format-ts");
const resizeConstraint_1 = require("./resizeConstraint");
const models_1 = require("./models");
const style_1 = require("./style");
exports.makeHorizontalPath = () => ({
    isClosed: false,
    points: [
        {
            _class: 'curvePoint',
            cornerRadius: 0,
            curveFrom: '{0, 0}',
            curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
            curveTo: '{0, 0}',
            hasCurveFrom: false,
            hasCurveTo: false,
            point: '{0, 0.5}',
        },
        {
            _class: 'curvePoint',
            cornerRadius: 0,
            curveFrom: '{0, 0}',
            curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
            curveTo: '{0, 0}',
            hasCurveFrom: false,
            hasCurveTo: false,
            point: '{1, 0.5}',
        },
    ],
});
exports.makeVerticalPath = () => ({
    isClosed: false,
    points: [
        {
            _class: 'curvePoint',
            cornerRadius: 0,
            curveFrom: '{0, 0}',
            curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
            curveTo: '{0, 0}',
            hasCurveFrom: false,
            hasCurveTo: false,
            point: '{0.5, 0}',
        },
        {
            _class: 'curvePoint',
            cornerRadius: 0,
            curveFrom: '{0, 0}',
            curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
            curveTo: '{0, 0}',
            hasCurveFrom: false,
            hasCurveTo: false,
            point: '{0.5, 1}',
        },
    ],
});
exports.makeRectPath = (radii = [0, 0, 0, 0]) => {
    const [r0, r1, r2, r3] = radii;
    return {
        isClosed: true,
        points: [
            {
                _class: 'curvePoint',
                cornerRadius: r0 || 0,
                curveFrom: '{0, 0}',
                curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
                curveTo: '{0, 0}',
                hasCurveFrom: false,
                hasCurveTo: false,
                point: '{0, 0}',
            },
            {
                _class: 'curvePoint',
                cornerRadius: r1 || 0,
                curveFrom: '{1, 0}',
                curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
                curveTo: '{1, 0}',
                hasCurveFrom: false,
                hasCurveTo: false,
                point: '{1, 0}',
            },
            {
                _class: 'curvePoint',
                cornerRadius: r2 || 0,
                curveFrom: '{1, 1}',
                curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
                curveTo: '{1, 1}',
                hasCurveFrom: false,
                hasCurveTo: false,
                point: '{1, 1}',
            },
            {
                _class: 'curvePoint',
                cornerRadius: r3 || 0,
                curveFrom: '{0, 1}',
                curveMode: sketch_file_format_ts_1.FileFormat1.CurveMode.Straight,
                curveTo: '{0, 1}',
                hasCurveFrom: false,
                hasCurveTo: false,
                point: '{0, 1}',
            },
        ],
    };
};
exports.makeShapePath = (frame, path, resizingConstraint) => (Object.assign(Object.assign({ _class: 'shapePath', frame, do_objectID: models_1.generateID(), isFlippedHorizontal: false, isFlippedVertical: false, isLocked: false, isVisible: true, layerListExpandedType: sketch_file_format_ts_1.FileFormat1.LayerListExpanded.Undecided, name: 'Path', nameIsFixed: false, resizingConstraint: resizeConstraint_1.makeResizeConstraint(resizingConstraint), resizingType: sketch_file_format_ts_1.FileFormat1.ResizeType.Stretch, rotation: 0, shouldBreakMaskChain: false, booleanOperation: sketch_file_format_ts_1.FileFormat1.BooleanOperation.NA, edited: false }, path), { isFixedToViewport: false, pointRadiusBehaviour: sketch_file_format_ts_1.FileFormat1.PointsRadiusBehaviour.Rounded, exportOptions: {
        _class: 'exportOptions',
        exportFormats: [],
        includedLayerIds: [],
        layerOptions: 0,
        shouldTrim: false,
    } }));
exports.makeRectShapeLayer = (x, y, width, height, radii = [0, 0, 0, 0], resizingConstraint) => {
    const fixedRadius = radii[0] || 0;
    const path = exports.makeRectPath(radii);
    return Object.assign(Object.assign({}, path), { _class: 'rectangle', do_objectID: models_1.generateID(), frame: models_1.makeRect(x, y, width, height), isFlippedHorizontal: false, isFlippedVertical: false, isLocked: false, isVisible: true, layerListExpandedType: sketch_file_format_ts_1.FileFormat1.LayerListExpanded.Undecided, name: 'Path', nameIsFixed: false, resizingConstraint: resizeConstraint_1.makeResizeConstraint(resizingConstraint), resizingType: sketch_file_format_ts_1.FileFormat1.ResizeType.Stretch, rotation: 0, shouldBreakMaskChain: false, booleanOperation: sketch_file_format_ts_1.FileFormat1.BooleanOperation.NA, edited: false, fixedRadius, hasConvertedToNewRoundCorners: true, isFixedToViewport: false, pointRadiusBehaviour: sketch_file_format_ts_1.FileFormat1.PointsRadiusBehaviour.Rounded, exportOptions: {
            _class: 'exportOptions',
            exportFormats: [],
            includedLayerIds: [],
            layerOptions: 0,
            shouldTrim: false,
        } });
};
exports.makeShapeGroup = (frame, layers = [], style, shadows, fills, resizingConstraint) => ({
    _class: 'shapeGroup',
    do_objectID: models_1.generateID(),
    frame,
    isLocked: false,
    isVisible: true,
    name: 'ShapeGroup',
    nameIsFixed: false,
    resizingConstraint: resizeConstraint_1.makeResizeConstraint(resizingConstraint),
    resizingType: sketch_file_format_ts_1.FileFormat1.ResizeType.Stretch,
    rotation: 0,
    shouldBreakMaskChain: false,
    style: style_1.makeStyle(style, fills, shadows),
    hasClickThrough: false,
    layers,
    clippingMaskMode: 0,
    hasClippingMask: false,
    windingRule: sketch_file_format_ts_1.FileFormat1.WindingRule.EvenOdd,
    isFixedToViewport: false,
    exportOptions: {
        _class: 'exportOptions',
        exportFormats: [],
        includedLayerIds: [],
        layerOptions: 0,
        shouldTrim: false,
    },
    isFlippedHorizontal: false,
    isFlippedVertical: false,
    booleanOperation: sketch_file_format_ts_1.FileFormat1.BooleanOperation.NA,
    layerListExpandedType: sketch_file_format_ts_1.FileFormat1.LayerListExpanded.Undecided,
});
exports.makeVerticalBorder = (x, y, length, thickness, color) => {
    const frame = models_1.makeRect(x, y, thickness, length);
    const shapeFrame = models_1.makeRect(0, 0, thickness, length);
    const shapePath = exports.makeShapePath(shapeFrame, exports.makeVerticalPath());
    const content = exports.makeShapeGroup(frame, [shapePath]);
    if (!content.style) {
        return content;
    }
    content.style.borders = [
        {
            _class: 'border',
            isEnabled: true,
            color: models_1.makeColorFromCSS(color),
            fillType: sketch_file_format_ts_1.FileFormat1.FillType.Color,
            position: sketch_file_format_ts_1.FileFormat1.BorderPosition.Center,
            thickness,
            contextSettings: {
                _class: 'graphicsContextSettings',
                blendMode: sketch_file_format_ts_1.FileFormat1.BlendMode.Normal,
                opacity: 1,
            },
            gradient: models_1.emptyGradient,
        },
    ];
    return content;
};
exports.makeHorizontalBorder = (x, y, length, thickness, color) => {
    const frame = models_1.makeRect(x, y, length, thickness);
    const shapeFrame = models_1.makeRect(0, 0, length, thickness);
    const shapePath = exports.makeShapePath(shapeFrame, exports.makeHorizontalPath());
    const content = exports.makeShapeGroup(frame, [shapePath]);
    if (!content.style) {
        return content;
    }
    content.style.borders = [
        {
            _class: 'border',
            isEnabled: true,
            color: models_1.makeColorFromCSS(color),
            fillType: sketch_file_format_ts_1.FileFormat1.FillType.Color,
            position: sketch_file_format_ts_1.FileFormat1.BorderPosition.Center,
            thickness,
            contextSettings: {
                _class: 'graphicsContextSettings',
                blendMode: sketch_file_format_ts_1.FileFormat1.BlendMode.Normal,
                opacity: 1,
            },
            gradient: models_1.emptyGradient,
        },
    ];
    return content;
};
