var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { FileFormat1 as FileFormat } from '@sketch-hq/sketch-file-format-ts';
import svgModel from '@lona/svg-model';
import { makeShapeGroup, makeShapePath } from '../shapeLayers';
import { makeRect } from '../models';
import { createUniformBorder } from '../borders';
import { layerGroup } from '../layerGroup';
import { makePathsFromCommands, makeLineCapStyle } from './graphics/path';
import { unionRects, scaleRect, makeBoundingRectFromCommands, resize } from './graphics/rect';
function makeLayerFromPathElement(pathElement, _parentFrame, scale) {
    var _a = pathElement.data.params, commands = _a.commands, style = _a.style;
    // Paths are created using the original frame
    var pathFrame = makeBoundingRectFromCommands(commands);
    var paths = makePathsFromCommands(commands, pathFrame);
    // Scale the frame to fill the viewBox
    var shapeGroupFrame = scaleRect(pathFrame, scale);
    // Each shape path has an origin of {0, 0}, since the shapeGroup layer stores the real origin,
    // and we don't want to apply the origin translation twice.
    var shapePathFrame = makeRect(0, 0, shapeGroupFrame.width, shapeGroupFrame.height);
    var shapePaths = paths.map(function (path) { return makeShapePath(shapePathFrame, path); });
    var viewStyle = {};
    if (style.fill) {
        viewStyle.backgroundColor = style.fill;
    }
    var shapeGroup = makeShapeGroup(shapeGroupFrame, shapePaths, viewStyle);
    if (style.stroke && shapeGroup.style) {
        var lineCap = makeLineCapStyle(style.strokeLineCap);
        var borderStyle = createUniformBorder(style.strokeWidth * scale, style.stroke, undefined, FileFormat.BorderPosition.Center, lineCap, lineCap);
        shapeGroup.style = __assign(__assign({}, shapeGroup.style), borderStyle);
    }
    return shapeGroup;
}
function makeLayerGroup(frame, layers, name) {
    var group = layerGroup(frame.x, frame.y, frame.width, frame.height, 1);
    group.name = name;
    group.layers = layers;
    return group;
}
export function makeSvgLayer(layout, name, svg) {
    var _a = svgModel(svg).data, params = _a.params, children = _a.children;
    var _b = params.viewBox, viewBox = _b === void 0 ? {
        x: layout.left,
        y: layout.top,
        width: layout.width,
        height: layout.height,
    } : _b;
    // Determine the rect to generate layers within
    var croppedRect = resize(viewBox, layout, 'contain');
    var scale = croppedRect.width / viewBox.width;
    // The top-level frame is the union of every path within
    var frame = unionRects.apply(void 0, children.map(function (pathElement) {
        return makeBoundingRectFromCommands(pathElement.data.params.commands);
    }));
    // Scale the frame to fill the viewBox
    var scaledFrame = scaleRect(frame, scale);
    var layers = children.map(function (element) {
        return makeLayerFromPathElement(element, scaledFrame, scale);
    });
    return makeLayerGroup(croppedRect, layers, name);
}
