var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import { FileFormat1 as FileFormat } from '@sketch-hq/sketch-file-format-ts';
import { makeColorFromCSS, makeColorFill } from './models';
import { hasAnyDefined } from '../utils/hasAnyDefined';
import { DEFAULT_BORDER_STYLE } from './borders';
var DEFAULT_SHADOW_COLOR = '#000';
var SHADOW_STYLES = [
    'shadowColor',
    'shadowOffset',
    'shadowOpacity',
    'shadowRadius',
    'shadowSpread',
    'textShadowColor',
    'textShadowOffset',
    'textShadowOpacity',
    'textShadowRadius',
    'textShadowSpread',
];
var makeDashPattern = function (style, width) {
    switch (style) {
        case 'dashed':
            return [width * 3, width * 3];
        case 'dotted':
            return [width, width];
        case 'solid':
            return [];
        default:
            return [];
    }
};
export var makeBorderOptions = function (style, width, lineCapStyle, lineJoinStyle) {
    if (lineCapStyle === void 0) { lineCapStyle = FileFormat.LineCapStyle.Butt; }
    if (lineJoinStyle === void 0) { lineJoinStyle = FileFormat.LineJoinStyle.Miter; }
    return ({
        _class: 'borderOptions',
        isEnabled: false,
        dashPattern: makeDashPattern(style, width),
        lineCapStyle: lineCapStyle,
        lineJoinStyle: lineJoinStyle,
    });
};
export var makeShadow = function (style) {
    var opacity = style.shadowOpacity !== undefined && style.shadowOpacity !== null
        ? style.shadowOpacity
        : 'textShadowOpacity' in style &&
            style.textShadowOpacity !== undefined &&
            style.textShadowOpacity !== null
            ? style.textShadowOpacity
            : 1;
    var color = style.shadowColor ||
        ('textShadowColor' in style && style.textShadowColor) ||
        DEFAULT_SHADOW_COLOR;
    var radius = style.shadowRadius !== undefined && style.shadowRadius !== null
        ? style.shadowRadius
        : 'textShadowRadius' in style &&
            style.textShadowRadius !== undefined &&
            style.textShadowRadius !== null
            ? style.textShadowRadius
            : 1;
    var spread = style.shadowSpread !== undefined && style.shadowSpread !== null
        ? style.shadowSpread
        : 'textShadowSpread' in style &&
            style.textShadowSpread !== undefined &&
            style.textShadowSpread !== null
            ? style.textShadowSpread
            : 1;
    var _a = style.shadowOffset || ('textShadowOffset' in style && style.textShadowOffset) || {}, offsetX = _a.width, offsetY = _a.height;
    if (!offsetX) {
        offsetX = 0;
    }
    if (!offsetY) {
        offsetY = 0;
    }
    var commonProps = {
        isEnabled: true,
        blurRadius: radius,
        color: makeColorFromCSS(color, opacity),
        contextSettings: {
            _class: 'graphicsContextSettings',
            blendMode: FileFormat.BlendMode.Normal,
            opacity: 1,
        },
        offsetX: offsetX,
        offsetY: offsetY,
        spread: spread,
    };
    if (style.shadowInner) {
        return __assign({ _class: 'innerShadow' }, commonProps);
    }
    return __assign({ _class: 'shadow' }, commonProps);
};
export var makeStyle = function (style, fills, shadowsProp) {
    var json = {
        _class: 'style',
        fills: [],
        miterLimit: 10,
        innerShadows: [],
        shadows: [],
        borderOptions: makeBorderOptions(DEFAULT_BORDER_STYLE, 0, 0, 0),
        startMarkerType: FileFormat.MarkerType.OpenArrow,
        endMarkerType: FileFormat.MarkerType.OpenArrow,
        windingRule: FileFormat.WindingRule.EvenOdd,
        colorControls: {
            _class: 'colorControls',
            isEnabled: false,
            brightness: 1,
            contrast: 1,
            hue: 1,
            saturation: 1,
        },
    };
    if (fills && fills.length) {
        json.fills = (json.fills || []).concat(fills);
    }
    if (!style) {
        return json;
    }
    if (style.opacity) {
        json.contextSettings = {
            _class: 'graphicsContextSettings',
            blendMode: FileFormat.BlendMode.Normal,
            opacity: style.opacity,
        };
    }
    if (style.backgroundColor) {
        var fill = makeColorFill(style.backgroundColor);
        (json.fills || []).unshift(fill);
    }
    if (hasAnyDefined(style, SHADOW_STYLES)) {
        var shadow = [makeShadow(style)];
        if (style.shadowInner) {
            json.innerShadows = shadow;
        }
        else {
            json.shadows = shadow;
        }
    }
    if (shadowsProp) {
        shadowsProp.forEach(function (shadowStyle) {
            if (!shadowStyle) {
                return;
            }
            var shadow = makeShadow(shadowStyle);
            if (shadowStyle.shadowInner) {
                (json.innerShadows || []).push(shadow);
            }
            else {
                (json.shadows || []).push(shadow);
            }
        });
    }
    return json;
};
export function parseStyle(json) {
    var style = {};
    if (json.contextSettings && json.contextSettings.opacity !== 1) {
        style.opacity = json.contextSettings.opacity;
    }
    if (json.fills &&
        json.fills.length > 0 &&
        json.fills[0].fillType === FileFormat.FillType.Color &&
        json.fills[0].isEnabled) {
        var color = json.fills[0].color;
        style.backgroundColor = "#" + Math.round(color.red * 255).toString(16) + Math.round(color.green * 255).toString(16) + Math.round(color.blue * 255).toString(16);
        if (color.alpha !== 1) {
            style.backgroundColor += "" + Math.round(color.alpha * 255).toString(16);
        }
    }
    if ((json.shadows && json.shadows.length > 0 && json.shadows[0].isEnabled) ||
        (json.innerShadows && json.innerShadows.length > 0 && json.innerShadows[0].isEnabled)) {
        var isNormalShadow = json.shadows && json.shadows.length > 0 && json.shadows[0].isEnabled;
        var shadow = isNormalShadow ? (json.shadows || [])[0] : (json.innerShadows || [])[0];
        style.shadowRadius = shadow.blurRadius;
        style.shadowSpread = shadow.spread;
        style.shadowOffset = {
            width: shadow.offsetX,
            height: shadow.offsetY,
        };
        var color = shadow.color;
        style.shadowColor = "#" + Math.round(color.red * 255).toString(16) + Math.round(color.green * 255).toString(16) + Math.round(color.blue * 255).toString(16);
        if (color.alpha !== 1) {
            style.shadowOpacity = color.alpha;
        }
        if (!isNormalShadow) {
            style.shadowInner = true;
        }
    }
    return style;
}
