var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var _a;
import { FileFormat1 as FileFormat } from '@sketch-hq/sketch-file-format-ts';
import { makeResizeConstraint } from './resizeConstraint';
import { generateID, makeColorFromCSS } from './models';
import { makeStyle, parseStyle } from './style';
export var TEXT_DECORATION_UNDERLINE = {
    none: FileFormat.UnderlineStyle.None,
    underline: FileFormat.UnderlineStyle.Underlined,
    double: 9,
    'line-through': 0,
};
export var TEXT_ALIGN = {
    auto: FileFormat.TextHorizontalAlignment.Left,
    left: FileFormat.TextHorizontalAlignment.Left,
    right: FileFormat.TextHorizontalAlignment.Right,
    center: FileFormat.TextHorizontalAlignment.Centered,
    justify: FileFormat.TextHorizontalAlignment.Justified,
};
var TEXT_ALIGN_REVERSE = (_a = {},
    _a[FileFormat.TextHorizontalAlignment.Natural] = 'left',
    _a[FileFormat.TextHorizontalAlignment.Right] = 'right',
    _a[FileFormat.TextHorizontalAlignment.Centered] = 'center',
    _a[FileFormat.TextHorizontalAlignment.Justified] = 'justify',
    _a);
export var TEXT_DECORATION_LINETHROUGH = {
    none: 0,
    underline: 0,
    double: 0,
    'line-through': 1,
};
export var TEXT_TRANSFORM = {
    uppercase: FileFormat.TextTransform.Uppercase,
    lowercase: FileFormat.TextTransform.Lowercase,
    initial: FileFormat.TextTransform.None,
    inherit: FileFormat.TextTransform.None,
    none: FileFormat.TextTransform.None,
    capitalize: FileFormat.TextTransform.None,
};
// this borrows heavily from react-native's RCTFont class
// thanks y'all
// https://github.com/facebook/react-native/blob/master/React/Views/RCTFont.mm
export var FONT_STYLES = {
    normal: false,
    italic: true,
    oblique: true,
};
var makeFontDescriptor = function (bridge) { return function (style) { return ({
    _class: 'fontDescriptor',
    attributes: {
        name: bridge.findFontName(style),
        size: style.fontSize || 14,
    },
}); }; };
var makeTextStyleAttributes = function (bridge) { return function (style) {
    return (__assign(__assign(__assign({ underlineStyle: style.textDecoration ? TEXT_DECORATION_UNDERLINE[style.textDecoration] || 0 : 0, strikethroughStyle: style.textDecoration
            ? TEXT_DECORATION_LINETHROUGH[style.textDecoration] || 0
            : 0, paragraphStyle: __assign({ _class: 'paragraphStyle', alignment: TEXT_ALIGN[style.textAlign || 'auto'], paragraphSpacing: style.paragraphSpacing || 0 }, (typeof style.lineHeight !== 'undefined' && style.lineHeight !== null
            ? {
                minimumLineHeight: style.lineHeight,
                maximumLineHeight: style.lineHeight,
                lineHeightMultiple: 1.0,
            }
            : {})) }, (typeof style.letterSpacing !== 'undefined' && style.letterSpacing !== null
        ? {
            kerning: style.letterSpacing,
        }
        : {})), (typeof style.textTransform !== 'undefined' && style.textTransform !== null
        ? {
            MSAttributedStringTextTransformAttribute: TEXT_TRANSFORM[style.textTransform] * 1,
        }
        : {})), { MSAttributedStringFontAttribute: makeFontDescriptor(bridge)(style), textStyleVerticalAlignmentKey: 0, MSAttributedStringColorAttribute: makeColorFromCSS(style.color || 'black') }));
}; };
var makeAttribute = function (bridge) { return function (node, location) { return ({
    _class: 'stringAttribute',
    location: location,
    length: node.content.length,
    attributes: makeTextStyleAttributes(bridge)(node.textStyles),
}); }; };
var makeAttributedString = function (bridge) { return function (textNodes) {
    var json = {
        _class: 'attributedString',
        string: '',
        attributes: [],
    };
    var location = 0;
    textNodes.forEach(function (node) {
        json.attributes.push(makeAttribute(bridge)(node, location));
        json.string += node.content;
        location += node.content.length;
    });
    return json;
}; };
export var makeTextStyle = function (bridge) { return function (style, shadows) {
    var json = makeStyle(style, undefined, shadows);
    json.textStyle = {
        _class: 'textStyle',
        encodedAttributes: makeTextStyleAttributes(bridge)(style),
        verticalAlignment: FileFormat.TextVerticalAlignment.Top,
    };
    return json;
}; };
export var parseTextStyle = function (json) {
    var style = parseStyle(json);
    if (json.textStyle) {
        var attr = json.textStyle.encodedAttributes;
        if (attr.underlineStyle) {
            style.textDecoration = attr.underlineStyle === 9 ? 'double' : 'underline';
        }
        if (attr.strikethroughStyle) {
            style.textDecoration = 'line-through';
        }
        if (attr.paragraphStyle &&
            attr.paragraphStyle.alignment &&
            TEXT_ALIGN_REVERSE[attr.paragraphStyle.alignment]) {
            style.textAlign = TEXT_ALIGN_REVERSE[attr.paragraphStyle.alignment];
        }
        if (attr.paragraphStyle && typeof attr.paragraphStyle.minimumLineHeight !== 'undefined') {
            style.lineHeight = attr.paragraphStyle.minimumLineHeight;
        }
        if (typeof attr.kerning !== 'undefined') {
            style.letterSpacing = attr.kerning;
        }
        var color = json.textStyle.encodedAttributes.MSAttributedStringColorAttribute;
        if (color) {
            style.color = "#" + Math.round(color.red * 255).toString(16) + Math.round(color.green * 255).toString(16) + Math.round(color.blue * 255).toString(16);
            if (color.alpha !== 1) {
                style.color += "" + Math.round(color.alpha * 255).toString(16);
            }
        }
        if (json.textStyle.encodedAttributes.MSAttributedStringTextTransformAttribute !==
            FileFormat.TextTransform.None) {
            style.textTransform =
                json.textStyle.encodedAttributes.MSAttributedStringTextTransformAttribute ===
                    FileFormat.TextTransform.Lowercase
                    ? 'lowercase'
                    : 'uppercase';
        }
        var font = json.textStyle.encodedAttributes.MSAttributedStringFontAttribute;
        style.fontSize = font.attributes.size;
        // we are cheating here, setting the name of the font instead of parsing
        // the family, weight and traits. react-sketchapp will handle it nevertheless
        style.fontFamily = font.attributes.name;
    }
    return style;
};
export var makeTextLayer = function (bridge) { return function (frame, name, textNodes, _style, resizingConstraint, shadows) { return ({
    _class: 'text',
    do_objectID: generateID("text:" + name + "-" + textNodes.map(function (node) { return node.content; }).join('')),
    frame: frame,
    isFlippedHorizontal: false,
    isFlippedVertical: false,
    isLocked: false,
    isVisible: true,
    layerListExpandedType: FileFormat.LayerListExpanded.Undecided,
    name: name,
    nameIsFixed: false,
    resizingConstraint: makeResizeConstraint(resizingConstraint),
    resizingType: FileFormat.ResizeType.Stretch,
    rotation: 0,
    shouldBreakMaskChain: false,
    attributedString: makeAttributedString(bridge)(textNodes),
    style: makeTextStyle(bridge)((textNodes[0] || { textStyles: {} }).textStyles, shadows),
    automaticallyDrawOnUnderlyingPath: false,
    dontSynchroniseWithSymbol: false,
    // NOTE(akp): I haven't fully figured out the meaning of glyphBounds
    glyphBounds: '',
    // glyphBounds: '{{0, 0}, {116, 17}}',
    lineSpacingBehaviour: FileFormat.LineSpacingBehaviour.ConsistentBaseline,
    textBehaviour: FileFormat.TextBehaviour.Fixed,
    booleanOperation: FileFormat.BooleanOperation.NA,
    exportOptions: {
        _class: 'exportOptions',
        exportFormats: [],
        includedLayerIds: [],
        layerOptions: 0,
        shouldTrim: false,
    },
    isFixedToViewport: false,
}); }; };
