import * as React from 'react';
import { fromSJSON } from './jsonUtils/sketchJson/fromSJSON';
import { buildTree } from './buildTree';
import { flexToSketchJSON } from './flexToSketchJSON';
import { resetLayer, resetDocument } from './resets';
import { injectSymbols } from './symbol';
import { RedBox } from './components/RedBox';
import { getDocumentDataFromContainer, getDocumentDataFromContext, getDocumentData, } from './utils/getDocument';
import { isNativeDocument } from './utils/isNativeDocument';
import { isNativePage } from './utils/isNativePage';
import { isNativeSymbolsPage } from './utils/isNativeSymbolsPage';
export var renderLayers = function (layers, container) {
    if (container.addLayers === undefined) {
        throw new Error(" React SketchApp cannot render into this layer. You may be trying to render into a layer that does not take children. Try rendering into a LayerGroup, Artboard, or Page.");
    }
    container.addLayers(layers);
    return container;
};
var getDefaultPage = function () {
    var doc = getDocumentDataFromContext(context);
    var currentPage = doc.currentPage();
    return isNativeSymbolsPage(currentPage) ? doc.addBlankPage() : currentPage;
};
var renderContents = function (bridge) { return function (tree, container) {
    var json = flexToSketchJSON(bridge)(tree);
    var layer = fromSJSON(json, '119');
    return renderLayers([layer], container);
}; };
var renderPage = function (bridge) { return function (tree, page) {
    var children = tree.children || [];
    // assume if name is set on this nested page, the intent is to overwrite
    // the name of the page it is getting rendered into
    if (tree.props.name) {
        page.setName(tree.props.name);
    }
    return children.map(function (child) { return renderContents(bridge)(child, page); });
}; };
var renderDocument = function (bridge) { return function (tree, documentData) {
    var initialPage = documentData.currentPage();
    var shouldRenderInitialPage = !isNativeSymbolsPage(initialPage);
    var children = tree.children || [];
    return children.map(function (child, i) {
        if (typeof child === 'string' || child.type !== 'sketch_page') {
            throw new Error('Document children must be of type Page');
        }
        var page = i === 0 && shouldRenderInitialPage ? initialPage : documentData.addBlankPage();
        return renderPage(bridge)(child, page);
    });
}; };
var renderTree = function (bridge) { return function (tree, _container) {
    if (tree.type === 'sketch_document') {
        if (_container && !isNativeDocument(_container)) {
            throw new Error('Cannot render a Document into a child of Document');
        }
        var doc = getDocumentData(_container);
        if (!doc) {
            return;
        }
        resetDocument(doc);
        return renderDocument(bridge)(tree, doc);
    }
    if (isNativeDocument(_container)) {
        throw new Error('You need to render a Document into Document');
    }
    if (tree.type === 'sketch_page') {
        if (_container && !isNativePage(_container)) {
            throw new Error('You need to render a Page into Page');
        }
    }
    var container = _container || getDefaultPage();
    resetLayer(container);
    return tree.type === 'sketch_page'
        ? renderPage(bridge)(tree, container)
        : renderContents(bridge)(tree, container);
}; };
export var render = function (bridge) { return function (element, container) {
    var nativeContainer;
    if (container && container.sketchObject) {
        nativeContainer = container.sketchObject;
    }
    else if (container) {
        nativeContainer = container;
    }
    // The Symbols page holds a special meaning within Sketch / react-sketchapp
    // and due to how `makeSymbol` works, we cannot render into it
    if (isNativeSymbolsPage(nativeContainer)) {
        throw Error('Cannot render into Symbols page');
    }
    try {
        var tree = buildTree(bridge)(element);
        injectSymbols(getDocumentDataFromContainer(nativeContainer));
        return renderTree(bridge)(tree, nativeContainer);
    }
    catch (err) {
        console.error(err);
        var tree = buildTree(bridge)(React.createElement(RedBox, { error: err }));
        return renderContents(bridge)(tree, nativeContainer);
    }
}; };
