import { getSketchVersion } from '../utils/getSketchVersion';
import { hashStyle } from '../utils/hashStyle';
import { getDocument } from '../utils/getDocument';
import { sharedTextStyles } from '../utils/sharedTextStyles';
import { makeTextStyle } from '../jsonUtils/textLayers';
import { pick } from '../utils/pick';
import { INHERITABLE_FONT_STYLES } from '../utils/constants';
var _styles = {};
var _byName = {};
export var TextStyles = function (getDefaultBridge) { return ({
    registerStyle: function (name, style, platformBridge) {
        if (platformBridge === void 0) { platformBridge = getDefaultBridge(); }
        var safeStyle = pick(style, INHERITABLE_FONT_STYLES);
        var hash = hashStyle(safeStyle);
        var sketchStyle = makeTextStyle(platformBridge)(safeStyle);
        var sharedObjectID = sharedTextStyles.addStyle(name, sketchStyle);
        // FIXME(gold): side effect :'(
        _byName[name] = hash;
        _styles[hash] = {
            cssStyle: safeStyle,
            name: name,
            sketchStyle: sketchStyle,
            sharedObjectID: sharedObjectID,
        };
    },
    create: function (styles, options, platformBridge) {
        var _this = this;
        if (options === void 0) { options = {}; }
        if (platformBridge === void 0) { platformBridge = getDefaultBridge(); }
        var clearExistingStyles = options.clearExistingStyles, document = options.document;
        var doc = getDocument(document);
        var sketchVersion = getSketchVersion();
        if (sketchVersion !== 'NodeJS' && sketchVersion < 50) {
            if (doc) {
                doc.showMessage('💎 Requires Sketch 50+ 💎');
            }
            return {};
        }
        sharedTextStyles.setDocument(doc);
        if (clearExistingStyles) {
            _styles = {};
            sharedTextStyles.setStyles([]);
        }
        Object.keys(styles).forEach(function (name) { return _this.registerStyle(name, styles[name], platformBridge); });
        return _styles;
    },
    resolve: function (style) {
        if (!style) {
            return undefined;
        }
        var safeStyle = pick(style, INHERITABLE_FONT_STYLES);
        var hash = hashStyle(safeStyle);
        return _styles[hash];
    },
    get: function (name, document) {
        var hash = _byName[name];
        var style = _styles[hash];
        if (style) {
            return style.cssStyle;
        }
        return sharedTextStyles.getStyle(name, document ? getDocument(document) : undefined);
    },
    clear: function () {
        _styles = {};
        sharedTextStyles.setStyles([]);
    },
    toJSON: function () {
        return Object.keys(_styles).map(function (k) { return ({
            _class: 'sharedStyle',
            do_objectID: _styles[k].sharedObjectID,
            name: _styles[k].name,
            value: _styles[k].sketchStyle,
        }); });
    },
    styles: function () {
        return _styles;
    },
}); };
