var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
import * as React from 'react';
import * as PropTypes from 'prop-types';
import { fromSJSON } from './jsonUtils/sketchJson/fromSJSON';
import { toSJSON } from './jsonUtils/sketchJson/toSJSON';
import { StyleSheet } from './stylesheet';
import { generateID } from './jsonUtils/models';
import { ViewStylePropTypes } from './components/ViewStylePropTypes';
import { ResizingConstraintPropTypes } from './components/ResizingConstraintPropTypes';
import { buildTree } from './buildTree';
import { flexToSketchJSON } from './flexToSketchJSON';
import { renderLayers } from './render';
import { resetLayer } from './resets';
import { getDocumentData } from './utils/getDocument';
import { getSketchVersion } from './utils/getSketchVersion';
var id = 0;
var nextId = function () { return ++id; };
var displayName = function (Component) {
    return Component.displayName || Component.name || "UnknownSymbol" + nextId();
};
var hasInitialized = false;
var symbolsRegistry = {};
var existingSymbols = [];
var layers = {};
function msListToArray(pageList) {
    var out = [];
    for (var i = 0; i < pageList.length; i++) {
        out.push(pageList[i]);
    }
    return out;
}
var getSymbolsPage = function (documentData) {
    return documentData.symbolsPageOrCreateIfNecessary();
};
function exists(x) {
    return !!x;
}
var getExistingSymbols = function (documentData) {
    if (!hasInitialized) {
        hasInitialized = true;
        var symbolsPage = getSymbolsPage(documentData);
        existingSymbols = msListToArray(symbolsPage.layers())
            .map(function (x) {
            var symbolJson = toSJSON(x);
            if (!symbolJson || symbolJson._class !== 'symbolMaster') {
                return undefined;
            }
            layers[symbolJson.symbolID] = x;
            return symbolJson;
        })
            .filter(exists);
        existingSymbols.forEach(function (symbolMaster) {
            if (symbolMaster._class !== 'symbolMaster')
                return;
            if (symbolMaster.name in symbolsRegistry)
                return;
            symbolsRegistry[symbolMaster.name] = symbolMaster;
        });
    }
    return existingSymbols;
};
export var injectSymbols = function (document) {
    if (getSketchVersion() === 'NodeJS') {
        console.error('Cannot inject symbols in NodeJS');
        return;
    }
    // if hasInitialized is false then makeSymbol has not yet been called
    // so we don't have anything to inject
    if (hasInitialized) {
        var documentData = getDocumentData(document);
        if (!documentData) {
            return;
        }
        var currentPage = documentData.currentPage();
        var symbolsPage = getSymbolsPage(documentData);
        var left_1 = 0;
        Object.keys(symbolsRegistry).forEach(function (key) {
            var symbolMaster = symbolsRegistry[key];
            symbolMaster.frame.y = 0;
            symbolMaster.frame.x = left_1;
            left_1 += symbolMaster.frame.width + 20;
            var newLayer = fromSJSON(symbolMaster, '119');
            layers[symbolMaster.symbolID] = newLayer;
        });
        // Clear out page layers to prepare for re-render
        resetLayer(symbolsPage);
        renderLayers(Object.keys(layers).map(function (k) { return layers[k]; }), symbolsPage);
        documentData.setCurrentPage(currentPage);
    }
};
var SymbolInstancePropTypes = {
    style: PropTypes.shape(ViewStylePropTypes),
    name: PropTypes.string,
    overrides: PropTypes.objectOf(PropTypes.oneOfType([PropTypes.string, PropTypes.func])),
    resizingConstraint: PropTypes.shape(__assign({}, ResizingConstraintPropTypes)),
};
export var createSymbolInstanceClass = function (symbolMaster) {
    var _a;
    return _a = /** @class */ (function (_super) {
            __extends(SymbolInstance, _super);
            function SymbolInstance() {
                return _super !== null && _super.apply(this, arguments) || this;
            }
            SymbolInstance.prototype.render = function () {
                return (React.createElement("sketch_symbolinstance", { symbolID: symbolMaster.symbolID, name: this.props.name || symbolMaster.name, style: StyleSheet.flatten(this.props.style), resizingConstraint: this.props.resizingConstraint, overrides: this.props.overrides }));
            };
            return SymbolInstance;
        }(React.Component)),
        _a.displayName = "SymbolInstance(" + symbolMaster.name + ")",
        _a.propTypes = SymbolInstancePropTypes,
        _a.symbolID = symbolMaster.symbolID,
        _a.masterName = symbolMaster.name,
        _a;
};
var SymbolMasterPropTypes = {
    style: PropTypes.shape(ViewStylePropTypes),
    name: PropTypes.string,
};
export var makeSymbol = function (bridge) { return function (Component, symbolProps, document) {
    if (!hasInitialized && getSketchVersion() !== 'NodeJS') {
        var documentData = getDocumentData(document);
        if (documentData) {
            getExistingSymbols(documentData);
        }
    }
    var masterName = (typeof symbolProps === 'string' ? symbolProps : (symbolProps || {}).name) ||
        displayName(Component);
    var existingSymbol = existingSymbols.find(function (symbolMaster) { return symbolMaster.name === masterName; });
    var symbolID = existingSymbol
        ? existingSymbol.symbolID
        : generateID("symbolID:" + masterName, !!masterName);
    var symbolMaster = flexToSketchJSON(bridge)(buildTree(bridge)(React.createElement("sketch_symbolmaster", __assign({}, (typeof symbolProps !== 'string' ? symbolProps || {} : {}), { symbolID: symbolID, name: masterName }),
        React.createElement(Component, null))));
    symbolsRegistry[symbolID] = symbolMaster;
    return createSymbolInstanceClass(symbolMaster);
}; };
function tryGettingSymbolMasterInDocumentByName(name, document) {
    var documentData = getDocumentData(document);
    if (!documentData) {
        return undefined;
    }
    var symbols = documentData.symbolMap();
    var symbol = Object.keys(symbols).find(function (k) { return symbols[k].name() === name; });
    if (!symbol) {
        return undefined;
    }
    return toSJSON(symbol);
}
function tryGettingSymbolMasterInDocumentById(symbolID, document) {
    var documentData = getDocumentData(document);
    if (!documentData) {
        return undefined;
    }
    var symbol = documentData.symbolMap()[symbolID];
    if (!symbol) {
        return undefined;
    }
    return toSJSON(symbol);
}
export var getSymbolMasterByName = function (name, document) {
    var symbolID = name
        ? Object.keys(symbolsRegistry).find(function (key) { return String(symbolsRegistry[key].name) === name; })
        : '';
    if (typeof symbolID === 'undefined' && name && getSketchVersion() !== 'NodeJS') {
        return tryGettingSymbolMasterInDocumentByName(name, document);
    }
    if (typeof symbolID === 'undefined') {
        throw new Error('##FIXME## NO MASTER FOR THIS SYMBOL NAME');
    }
    return symbolsRegistry[symbolID];
};
export var getSymbolMasterById = function (symbolID, document) {
    var symbolMaster = symbolID ? symbolsRegistry[symbolID] : undefined;
    if (typeof symbolMaster === 'undefined' && symbolID && getSketchVersion() !== 'NodeJS') {
        symbolMaster = tryGettingSymbolMasterInDocumentById(symbolID, document);
    }
    if (typeof symbolMaster === 'undefined') {
        throw new Error('##FIXME## NO MASTER WITH THAT SYMBOL ID');
    }
    return symbolMaster;
};
export var getSymbolComponentById = function (symbolID, document) {
    var symbolMaster = getSymbolMasterById(symbolID, document);
    if (!symbolMaster) {
        return undefined;
    }
    return createSymbolInstanceClass(symbolMaster);
};
export var getSymbolComponentByName = function (masterName, document) {
    var symbolMaster = getSymbolMasterByName(masterName, document);
    if (!symbolMaster) {
        return undefined;
    }
    return createSymbolInstanceClass(symbolMaster);
};
