import * as PropTypes from 'prop-types';
import { ViewStylePropTypes, Color as ColorProp, BorderStyle as BorderProp, Overflow as OverflowProp } from '../components/ViewStylePropTypes';
import { TextStylePropTypes } from '../components/TextStylePropTypes';
export declare type SketchLayer = any;
export declare type WrappedSketchLayer = {
    sketchObject: SketchLayer;
};
export declare type MSArray<T> = {
    [key: number]: T;
    length: number;
};
declare type NSString = any;
export declare type SketchPage = {
    name: () => NSString;
    setName: (name: string) => void;
    layers: () => Array<SketchLayer>;
    children: () => Array<SketchLayer>;
};
export declare type SketchStyle = any;
export declare type SketchSharedStyleContainer = {
    objects: () => any[];
    setObjects: (objects: Array<SketchStyle>) => void;
    addSharedStyleWithName_firstInstance: (name: string, ins: SketchStyle) => any;
    addSharedObject: (ins: any) => any;
};
declare type MSGradient = any;
declare type MSColor = any;
declare type SketchAssetCollection = {
    colors: () => Array<MSColor>;
    gradients: () => Array<MSGradient>;
};
export declare type SketchDocumentData = {
    delegate: () => SketchDocument;
    assets: () => SketchAssetCollection;
    layerStyles: () => void;
    layerTextStyles: () => SketchSharedStyleContainer;
    layerSymbols: () => void;
    symbolMap: () => {
        [symbolID: string]: SketchLayer;
    };
    removePageAtIndex: (index: number) => void;
    addBlankPage: () => SketchPage;
    currentPage: () => SketchPage;
    setCurrentPage: (page: SketchPage) => void;
    pages: () => MSArray<SketchPage>;
    symbolsPageOrCreateIfNecessary: () => SketchPage;
};
export declare type SketchDocument = {
    documentData: () => SketchDocumentData;
    showMessage: (message: string) => void;
};
export declare type WrappedSketchDocument = {
    sketchObject: SketchDocument | SketchDocumentData;
};
export declare type SketchContext = {
    document: SketchDocument;
    actionContext: {
        document?: SketchDocument;
    };
};
export declare type Size = {
    width: number;
    height: number;
};
export declare type LayoutInfo = {
    width: number;
    height: number;
    top: number;
    left: number;
    right: number;
    bottom: number;
    direction?: 'ltr' | 'rtl';
};
export declare type MeasureMode = 'undefined' | 'exactly' | 'at-most';
export declare type Color = PropTypes.InferType<typeof ColorProp>;
export declare type BorderStyle = PropTypes.InferType<typeof BorderProp>;
export declare type Overflow = PropTypes.InferType<typeof OverflowProp>;
export declare type ViewStyle = PropTypes.InferProps<typeof ViewStylePropTypes> & {
    color?: string;
};
export declare type TextStyle = PropTypes.InferProps<typeof TextStylePropTypes> & {
    color?: string;
};
export declare type TextNode = {
    content: string;
    textStyles: TextStyle;
};
export declare type TreeNode<Props = any> = {
    type: string;
    style: ViewStyle;
    textStyle?: TextStyle;
    layout: LayoutInfo;
    props: Props & {
        textNodes: TextNode[];
    };
    children?: Array<TreeNode | string>;
};
export declare type ResizeConstraints = {
    top?: boolean;
    right?: boolean;
    bottom?: boolean;
    left?: boolean;
    fixedHeight?: boolean;
    fixedWidth?: boolean;
};
export declare type PlatformBridge = {
    createStringMeasurer(textNodes: TextNode[], maxWidth: number): Size;
    findFontName(style: TextStyle): string;
    makeImageDataFromUrl(url?: string): string;
};
export {};
