"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createStringMeasurer = void 0;
const textLayers_1 = require("../../jsonUtils/textLayers");
const models_1 = require("../../jsonUtils/models");
const findFontName_1 = require("./findFontName");
// TODO(lmr): do something more sensible here
const FLOAT_MAX = 999999;
function makeParagraphStyle(textStyle) {
    const pStyle = NSMutableParagraphStyle.alloc().init();
    if (textStyle.lineHeight !== undefined) {
        pStyle.minimumLineHeight = textStyle.lineHeight;
        pStyle.lineHeightMultiple = 1.0;
        pStyle.maximumLineHeight = textStyle.lineHeight;
    }
    if (textStyle.textAlign) {
        pStyle.alignment = textLayers_1.TEXT_ALIGN[textStyle.textAlign];
    }
    // TODO: check against only positive spacing values?
    if (textStyle.paragraphSpacing !== undefined) {
        pStyle.paragraphSpacing = textStyle.paragraphSpacing;
    }
    return pStyle;
}
// This shouldn't need to call into Sketch, but it does currently, which is bad for perf :(
function createStringAttributes(textStyles) {
    const font = findFontName_1.findFont(textStyles);
    const { textDecoration } = textStyles;
    const underline = textDecoration && textLayers_1.TEXT_DECORATION_UNDERLINE[textDecoration];
    const strikethrough = textDecoration && textLayers_1.TEXT_DECORATION_LINETHROUGH[textDecoration];
    const attribs = {
        MSAttributedStringFontAttribute: font.fontDescriptor(),
        NSFont: font,
        NSParagraphStyle: makeParagraphStyle(textStyles),
        NSUnderline: underline || 0,
        NSStrikethrough: strikethrough || 0,
    };
    const color = models_1.makeColorFromCSS(textStyles.color || 'black');
    attribs.MSAttributedStringColorAttribute = color;
    if (textStyles.letterSpacing !== undefined && textStyles.letterSpacing !== null) {
        attribs.NSKern = textStyles.letterSpacing;
    }
    if (textStyles.textTransform !== undefined && textStyles.textTransform !== null) {
        attribs.MSAttributedStringTextTransformAttribute = textLayers_1.TEXT_TRANSFORM[textStyles.textTransform] * 1;
    }
    return attribs;
}
function createAttributedString(textNode) {
    const { content, textStyles } = textNode;
    const attribs = createStringAttributes(textStyles);
    return NSAttributedString.attributedStringWithString_attributes_(content, attribs);
}
function createStringMeasurer(textNodes, width) {
    const fullStr = NSMutableAttributedString.alloc().init();
    textNodes.forEach((textNode) => {
        const newString = createAttributedString(textNode);
        fullStr.appendAttributedString(newString);
    });
    const { height: measureHeight, width: measureWidth, } = fullStr.boundingRectWithSize_options_context(CGSizeMake(width, FLOAT_MAX), NSStringDrawingUsesLineFragmentOrigin, null).size;
    return { width: measureWidth, height: measureHeight };
}
exports.createStringMeasurer = createStringMeasurer;
