"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = exports.renderLayers = void 0;
const React = __importStar(require("react"));
const fromSJSON_1 = require("./jsonUtils/sketchJson/fromSJSON");
const buildTree_1 = require("./buildTree");
const flexToSketchJSON_1 = require("./flexToSketchJSON");
const resets_1 = require("./resets");
const symbol_1 = require("./symbol");
const RedBox_1 = require("./components/RedBox");
const getDocument_1 = require("./utils/getDocument");
const isNativeDocument_1 = require("./utils/isNativeDocument");
const isNativePage_1 = require("./utils/isNativePage");
const isNativeSymbolsPage_1 = require("./utils/isNativeSymbolsPage");
exports.renderLayers = (layers, container) => {
    if (container.addLayers === undefined) {
        throw new Error(` React SketchApp cannot render into this layer. You may be trying to render into a layer that does not take children. Try rendering into a LayerGroup, Artboard, or Page.`);
    }
    container.addLayers(layers);
    return container;
};
const getDefaultPage = () => {
    const doc = getDocument_1.getDocumentDataFromContext(context);
    const currentPage = doc.currentPage();
    return isNativeSymbolsPage_1.isNativeSymbolsPage(currentPage) ? doc.addBlankPage() : currentPage;
};
const renderContents = (bridge) => (tree, container) => {
    const json = flexToSketchJSON_1.flexToSketchJSON(bridge)(tree);
    const layer = fromSJSON_1.fromSJSON(json, '119');
    return exports.renderLayers([layer], container);
};
const renderPage = (bridge) => (tree, page) => {
    const children = tree.children || [];
    // assume if name is set on this nested page, the intent is to overwrite
    // the name of the page it is getting rendered into
    if (tree.props.name) {
        page.setName(tree.props.name);
    }
    return children.map((child) => renderContents(bridge)(child, page));
};
const renderDocument = (bridge) => (tree, documentData) => {
    const initialPage = documentData.currentPage();
    const shouldRenderInitialPage = !isNativeSymbolsPage_1.isNativeSymbolsPage(initialPage);
    const children = tree.children || [];
    return children.map((child, i) => {
        if (typeof child === 'string' || child.type !== 'sketch_page') {
            throw new Error('Document children must be of type Page');
        }
        const page = i === 0 && shouldRenderInitialPage ? initialPage : documentData.addBlankPage();
        return renderPage(bridge)(child, page);
    });
};
const renderTree = (bridge) => (tree, _container) => {
    if (tree.type === 'sketch_document') {
        if (_container && !isNativeDocument_1.isNativeDocument(_container)) {
            throw new Error('Cannot render a Document into a child of Document');
        }
        const doc = getDocument_1.getDocumentData(_container);
        if (!doc) {
            return;
        }
        resets_1.resetDocument(doc);
        return renderDocument(bridge)(tree, doc);
    }
    if (isNativeDocument_1.isNativeDocument(_container)) {
        throw new Error('You need to render a Document into Document');
    }
    if (tree.type === 'sketch_page') {
        if (_container && !isNativePage_1.isNativePage(_container)) {
            throw new Error('You need to render a Page into Page');
        }
    }
    const container = _container || getDefaultPage();
    resets_1.resetLayer(container);
    return tree.type === 'sketch_page'
        ? renderPage(bridge)(tree, container)
        : renderContents(bridge)(tree, container);
};
exports.render = (bridge) => (element, container) => {
    let nativeContainer;
    if (container && container.sketchObject) {
        nativeContainer = container.sketchObject;
    }
    else if (container) {
        nativeContainer = container;
    }
    // The Symbols page holds a special meaning within Sketch / react-sketchapp
    // and due to how `makeSymbol` works, we cannot render into it
    if (isNativeSymbolsPage_1.isNativeSymbolsPage(nativeContainer)) {
        throw Error('Cannot render into Symbols page');
    }
    try {
        const tree = buildTree_1.buildTree(bridge)(element);
        symbol_1.injectSymbols(getDocument_1.getDocumentDataFromContainer(nativeContainer));
        return renderTree(bridge)(tree, nativeContainer);
    }
    catch (err) {
        console.error(err);
        const tree = buildTree_1.buildTree(bridge)(React.createElement(RedBox_1.RedBox, { error: err }));
        return renderContents(bridge)(tree, nativeContainer);
    }
};
