"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resetDocument = exports.resetLayer = void 0;
const isNativeDocument_1 = require("./utils/isNativeDocument");
const isNativeSymbolsPage_1 = require("./utils/isNativeSymbolsPage");
exports.resetLayer = (container) => {
    if (isNativeDocument_1.isNativeDocument(container)) {
        exports.resetDocument(container);
        return;
    }
    const layers = container.children();
    // Skip last child since it is the container itself
    for (let l = 0; l < layers.length - 1; l += 1) {
        const layer = layers[l];
        layer.removeFromParent();
    }
};
// Clear out all document pages and layers
exports.resetDocument = (documentData) => {
    // Get Pages and delete them all (Except Symbols Page)
    const pages = documentData.pages();
    for (let index = pages.length - 1; index >= 0; index -= 1) {
        const page = pages[index];
        // Don't delete symbols page
        if (!isNativeSymbolsPage_1.isNativeSymbolsPage(page)) {
            if (pages.length > 1) {
                documentData.removePageAtIndex(index);
            }
            else {
                exports.resetLayer(page);
            }
        }
    }
};
