"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processTransform = void 0;
const parseTransformProp_1 = require("./parseTransformProp");
const parseTransformOriginProp_1 = require("./parseTransformOriginProp");
function closeEnough(a, b) {
    return Math.abs(a - b) < 0.01;
}
function isRotation(a, b, c, d) {
    return closeEnough(a, d) && closeEnough(c, -b) && closeEnough(a * d - c * b, 1);
}
const rad2deg = 180 / Math.PI;
function getRotation(a, b) {
    const sketchFactor = -1;
    const possibleRotation = Math.acos(a);
    if (closeEnough(Math.sin(possibleRotation), b) || closeEnough(Math.sin(possibleRotation), -b)) {
        return sketchFactor * possibleRotation * rad2deg;
    }
    return sketchFactor * (possibleRotation + Math.PI) * rad2deg;
}
function processTransform(layout, props) {
    if (!props.transform) {
        return {};
    }
    const origin = parseTransformOriginProp_1.parseTransformOriginProp(layout, props.transformOrigin);
    const [a, b, c, d, tx, ty] = parseTransformProp_1.parseTransformProp(props.transform, origin);
    // apply translation
    layout.top += ty;
    layout.left += tx;
    // look for a rotation
    if (isRotation(a, b, c, d)) {
        return {
            rotation: getRotation(a, b),
        };
    }
    // let's try to check if there is a reflection
    // we are going to apply the same reflection and see if the result is a rotation
    /**
     * check if flipped vertically
     * 1  0
     * 0 -1
     */
    let _a = a;
    let _b = -b;
    let _c = c;
    let _d = -d;
    if (isRotation(_a, _b, _c, _d)) {
        return {
            rotation: getRotation(_a, _b),
            isFlippedVertical: true,
        };
    }
    /**
     * check if flipped horizontally
     * -1 0
     *  0 1
     */
    _a = -a;
    _b = b;
    _c = -c;
    _d = d;
    if (isRotation(_a, _b, _c, _d)) {
        return {
            rotation: getRotation(_a, _b),
            isFlippedHorizontal: true,
        };
    }
    /**
     * no need to check if flipped vertically and horizontally since it's a rotation
     */
    // didn't find any rotation or reflection
    return {};
}
exports.processTransform = processTransform;
